#!/usr/bin/python3

import re

class Config:

  path: str = ''
  ELASTIC_URL: str = 'http://localhost:9200'

  def __init__(self,
    ES_URLS: str = [],
    MAX_CURRENT_INDEX_SIZE_GB: int = 50,
    MAX_RELOCATING: int = 25,
    MAX_TASKS: int = 500,
    MASTER_TIMEOUT: str = '2m',
    HOT_DELAYED_TIMEOUT: str = '10m',
    COLD_DELAYED_TIMEOUT: str = '25m',
    SERVICE_INDEX: str = '.feii_ilm',
    SERVICE_INDEX_EXCEPTION: str = '.feii_ilm_exception',
    index_pattern: str = re.compile(r'^(shrink-)?(.+)-(\d{6})$'),
    index_pools: str = '',
    alias_list: str = '',
    settings_list: str = '',
    ilm_list: str = '',
    shards_list: str = '',
    ilm_policy: str = ''
  ):
    self.ES_URLS: str = [
      "{0}/_cat/indices?format=json&bytes=gb".format(self.ELASTIC_URL),
      "{0}/_alias".format(self.ELASTIC_URL),
      "{0}/*/_settings".format(self.ELASTIC_URL),
      "{0}/*/_ilm/explain".format(self.ELASTIC_URL),
      "{0}/_cat/shards/*?format=json&h=index,prirep,state,unassigned.reason,unassigned.details".format(self.ELASTIC_URL),
      "{0}/_ilm/policy".format(self.ELASTIC_URL),
    ]
    self.MAX_CURRENT_INDEX_SIZE_GB = MAX_CURRENT_INDEX_SIZE_GB
    self.MAX_RELOCATING = MAX_RELOCATING
    self.MAX_TASKS = MAX_TASKS
    self.MASTER_TIMEOUT = MASTER_TIMEOUT
    self.HOT_DELAYED_TIMEOUT = HOT_DELAYED_TIMEOUT
    self.COLD_DELAYED_TIMEOUT = COLD_DELAYED_TIMEOUT
    self.SERVICE_INDEX = SERVICE_INDEX
    self.SERVICE_INDEX_EXCEPTION = SERVICE_INDEX_EXCEPTION
    self.index_pattern = index_pattern
    self.index_pools = index_pools
    self.alias_list = alias_list
    self.settings_list = settings_list
    self.ilm_list = ilm_list
    self.shards_list = shards_list
    self.ilm_policy = ilm_policy



#!/usr/bin/python3

from feii.config import Config
from feii.request import Request

class Shard(Config, Request):
  def __init__(self,
    shards: str = [],
    shard_details: str = [],
    shards_details: str = [],
    shard_to_remove: str = [],
    unassigned_shards: str = [],
  ):
    super().__init__()
    self.shards = shards
    self.shard_details = shard_details
    self.shards_details = shards_details
    self.shard_to_remove = shard_to_remove
    self.unassigned_shards = unassigned_shards

  def debug_detail_index(self):
    self.alias = 'test'
    self.index = 'test-000001'

  def creating_array_shard_details_in_primary(self):
    for details in Config.index_pools[4].json():
      if details['prirep'] == 'p':
        self.shards_details.append(details)

  def creating_array_shard_not_duplicates(self):
    seen = set()
    for details in self.shards_details:
      if tuple(details.items()) not in seen:
        seen.add(tuple(details.items()))
        self.shard_details.append(details)

  def creating_array_shard_to_remove(self):
    for details in self.shard_details:
      if not self.index_pattern.match(details['index']):
        self.shard_to_remove.append(details)
        self.logger.warning("[{0}] encountered a strange index name".format(details['index']))

  def remove_invalid_shard_name_in_array(self):
    for details in self.shard_to_remove:
      self.shard_details.remove(details)

  def creating_array_shards(self):
    for details in self.shard_details:
      shard_details = details.copy()
      shard_details['number'] = int(self.index_pattern.match(details['index']).group(3))
      shard_details['index.alias'] = self.index_pattern.match(details['index']).group(2)
      shard_details['index.name'] = self.index_pattern.match(details['index']).group(2) + '-' + self.index_pattern.match(details['index']).group(3)
      self.shards.append(shard_details)

  def creating_array_unassigned_shard(self):
    for shard in self.shards:
      if shard['state'] == 'UNASSIGNED':
        self.unassigned_shards.append(shard)



#!/usr/bin/python3

import requests
from multiprocessing.dummy import Pool as ThreadPool
from feii.config import Config

class Init(Config):
  def __init__(self,
    count: int = 1,
  ):
    super().__init__()
    self.count = count

  def list_pools(self):
    pool = ThreadPool(int(self.count))
    results = pool.map(requests.get, self.ES_URLS)
    pool.close()
    return results



#!/usr/bin/python3

import re
import requests
from feii.config import Config
from feii.structure import Structure

class Delete(Structure):
  def __init__(self,
    data_are_in_index: bool = False
  ):
    super().__init__()
    self.data_are_in_index = data_are_in_index

  def debug_detail_index(self):
    self.index = 'test-000001'
    self.index_docs_count = 0

  def delete_not_last_indexes(self):
    for index in self.not_last_indices:
      self.index = index['index']
      if index['docs.count'] is None:
        continue

      if int(index['docs.count']) > 0:
        continue

      self.not_delete_index_and_check()

  def delete_not_last_indexes_check_mode(self):
    for index in self.not_last_indices:
      if index['docs.count'] is None:
        continue

      if int(index['docs.count']) > 0:
        continue

      self.logger.warning("[check_mode] Deletion an empty not last index [{0}], documents count - {1}".format( index['index'], index['docs.count'] ))

  def delete_last_indexes(self):
    for delete_index in self.delete_last_indices:
      self.index = delete_index['index']
      self.data_are_in_index = False

      if delete_index['docs.count'] is None:
        continue

      if int(delete_index['docs.count']) > 0:
        continue

      self.check_prev_indices()

      if not self.data_are_in_index:
        self.index = delete_index['index']
        self.creating_array_current_index()
        self.delete_current_indexes()

  def delete_last_indexes_check_mode(self):
    for delete_index in self.delete_last_indices:
      self.index = delete_index['index']
      self.data_are_in_index = False

      if delete_index['docs.count'] is None:
        continue

      if int(delete_index['docs.count']) > 0:
        continue

      self.check_prev_indices()

      if not self.data_are_in_index:
        self.logger.warning("[check_mode] Deletion an empty last index [{0}], documents count - {1}".format( delete_index['index'], delete_index['docs.count'] ))

  def delete_current_indexes(self):
    for index in self.current_indices[:]:
      self.index = index
      if not self.not_delete_index_and_check():
        self.current_indices.remove(index)

  def check_prev_indices(self):
    for x in range(int(self.index[-6:]) - 1):
      self.find_prev_index()
      self.without_shrink_prev_index = re.sub(r'(shrink-)', '', self.prev_index)

      if self.without_shrink_prev_index in Config.alias_list:
        self.creating_array_not_shrink_prev_index()

      if 'shrink-' + self.without_shrink_prev_index in Config.alias_list:
        self.creating_array_shrink_prev_index()

      self.check_empty_docs_in_prev_index()
      self.check_count_docs_in_prev_index()

      if not self.without_shrink_prev_index in Config.alias_list:
        self.index = self.without_shrink_prev_index
        continue

      if not 'shrink-' + self.without_shrink_prev_index in Config.alias_list:
        self.index = self.without_shrink_prev_index
        continue

  def check_empty_docs_in_prev_index(self):
    if 'docs.count' in self.prev_index_to_check and self.prev_index_to_check['docs.count'] is None:
      self.data_are_in_index = True

  def check_count_docs_in_prev_index(self):
    if 'docs.count' in self.prev_index_to_check and self.prev_index_to_check['docs.count'] != None and int(self.prev_index_to_check['docs.count']) > 0:
      self.data_are_in_index = True

  def delete_expired_policy_indices(self):
    self.check_service_index()

    self.creating_array_age_ilm_policy()

    self.creating_array_index_with_age()
    self.update_array_index_with_age()

    self.creating_array_index_exception()
    self.creating_array_index_to_expired_policy()

    self.delete_indexes_large_age()
    self.update_array_index_with_age()

    self.update_doc_to_service_index()

    self.checking_for_empty_doc_to_service_index()
    self.deleting_doc_to_service_index()

  def delete_expired_policy_indices_check_mode(self):
    self.creating_array_age_ilm_policy()

    self.creating_array_index_with_age()
    self.update_array_index_with_age()

    self.creating_array_index_exception()
    self.creating_array_index_to_expired_policy()

    for index in self.list_indexes_to_delete:
      self.logger.warning("[check_mode] Deletion expired ilm policy index [{0}].".format( index ))

  def delete_indexes_large_age(self):
    for index in self.list_indexes_to_delete:
      self.index = index
      if self.not_delete_index_and_check() != False:
        Config.ilm_list['indices'].pop(index)

  def deleting_doc_to_service_index(self):
    for deleted_document in self.full_deleted_indexes:
      self.document_id = deleted_document['_id']
      self.request = requests.delete("{0}/{1}/_doc/{2}?timeout={3}".format( self.ELASTIC_URL,  self.SERVICE_INDEX, self.document_id, self.MASTER_TIMEOUT ))
      self.check_deleting_doc_to_service_index()

  def check_deleting_doc_to_service_index(self):
    if self.status_request():
      self.logger.info("The document [{0}] has been removed from the service index [{1}]".format( self.document_id, self.SERVICE_INDEX))
      return True



#!/usr/bin/python3

import click
from feii.cli import AliasCLI

@click.command(
  cls=AliasCLI,
  short_help='Fix indexes without an alias.',
  epilog="Run 'feii alias COMMAND --help' for more information on a command."
)
def cli():
  """Fix indexes without an alias"""



#!/usr/bin/python3

import click
from feii.main import class_structure, class_log, logging_level, updating_variables, generating_variables_for_indices, generating_variables_for_alias, generating_variables_for_alias_not_shrink, deleting_unnecessary_variables
from feii.aliases import Aliases

class_aliases = Aliases()
class_aliases.logger = class_log.logger

def start_add_necessary_alias_for_indices(check_mode):
  if not check_mode:
    class_aliases.add_necessary_alias_for_indices()

def start_check_mode_add_necessary_alias_for_indices(check_mode):
  if check_mode:
    class_aliases.add_necessary_alias_for_indices_check_mode()

@click.command(short_help='Adding an primary alias for all indexes.')
@click.option(
  '-c', '--check-mode',
  is_flag=True,
  expose_value=True,
  help='Only displaying actions, without performing them.'
)
@click.option(
  '-l', '--log-level',
  default='info',
  show_default=True,
  expose_value=True,
  help='Set the logging level ("debug"|"info"|"warning"|"error"|"critical")'
)
@click.option(
  '-n', '--not-shrink',
  is_flag=True,
  expose_value=True,
  help='Indexes only, no shrink.'
)
@click.option(
  '-P', '--path-to-file',
  default='',
  expose_value=True,
  help='path'
)
def cli(check_mode, log_level, not_shrink, path_to_file):
  """Adding a primary alias for all indexes"""

  logging_level(log_level)
  class_log.logger.info("Started adding a primary alias for all indexes")

  updating_variables(path_to_file)

  generating_variables_for_indices()
  if not_shrink:
    generating_variables_for_alias_not_shrink()
  else:
    generating_variables_for_alias()
  deleting_unnecessary_variables()

  start_add_necessary_alias_for_indices(check_mode)
  start_check_mode_add_necessary_alias_for_indices(check_mode)

if __name__ == "__main__":
  cli()



#!/usr/bin/python3

import time

class Function:
  def __init__(self,
    next_index: str = '',
    prev_index: str = '',
    retry: int = 0,
  ):
    super().__init__()
    self.next_index = next_index
    self.prev_index = prev_index
    self.retry = retry

  def debug_detail_index(self):
    self.index = 'test-000001'
    self.retry = 1

  def find_next_index(self):
    alias = self.index[:-6]
    number = self.index[-6:]
    self.next_index = alias + str('{:06}'.format(int(number) + 1))
    return self.next_index

  def find_prev_index(self):
    alias = self.index[:-6]
    number = self.index[-6:]
    self.prev_index = alias + str('{:06}'.format(int(number) - 1))
    return self.prev_index

  def time_sleep(self):
    if self.retry > 0:
      self.logger.warning("Sleep time {0}".format( int(60 * self.retry) ))
      time.sleep(60 * self.retry)
